# (c) Copyright 2018 ZTE Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating Action via the DB API"""

import copy
from unittest import mock
from unittest.mock import patch

from oslo_config import cfg

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api.tests.unit import common
from freezer_api.tests.unit.sqlalchemy import base

CONF = cfg.CONF


class DbActionTestCase(base.DbTestCase):

    def setUp(self):
        super(DbActionTestCase, self).setUp()
        self.fake_action_0 = common.get_fake_action_0()
        self.fake_action_2 = common.get_fake_action_2()
        self.fake_action_3 = common.get_fake_action_3()
        self.freezer_action_0 = self.fake_action_0.get('freezer_action')
        self.freezer_action_2 = self.fake_action_2.get('freezer_action')
        self.fake_user_id = self.fake_action_0.get('user_id')
        self.fake_action_id = common.get_fake_action_id()
        CONF.enable_v1_api = True

    def tearDown(self):
        super(DbActionTestCase, self).tearDown()
        CONF.enable_v1_api = False

    def test_add_and_get_action(self):
        action_doc = copy.deepcopy(self.fake_action_0)
        action_id = self.dbapi.add_action(user_id=self.fake_action_0.
                                          get('user_id'),
                                          doc=action_doc)
        self.assertIsNotNone(action_id)

        result = self.dbapi.get_action(user_id=self.fake_action_0.
                                       get('user_id'),
                                       action_id=action_id)

        self.assertIsNotNone(result)

        self.assertEqual(result.get('max_retries'),
                         self.fake_action_0.get('max_retries'))

        self.assertEqual(result.get('max_retries_interval'),
                         self.fake_action_0.get('max_retries_interval'))

        freezer_action = result.get('freezer_action')

        self.assertEqual(freezer_action.get('action'),
                         self.freezer_action_0.get('action'))

        self.assertEqual(freezer_action.get('backup_name'),
                         self.freezer_action_0.get('backup_name'))

        self.assertEqual(freezer_action.get('container'),
                         self.freezer_action_0.get('container'))

        self.assertEqual(freezer_action.get('path_to_backup'),
                         self.freezer_action_0.get('path_to_backup'))

        self.assertEqual(freezer_action.get('mode'),
                         self.freezer_action_0.get('mode'))

    def test_add_and_delete_action(self):
        action_doc = copy.deepcopy(self.fake_action_0)
        action_id = self.dbapi.add_action(user_id=self.fake_action_0.
                                          get('user_id'),
                                          doc=action_doc)
        self.assertIsNotNone(action_id)

        result = self.dbapi.delete_action(user_id=self.fake_action_0.
                                          get('user_id'),
                                          action_id=action_id)

        self.assertIsNotNone(result)

        self.assertEqual(result, action_id)

        result = self.dbapi.get_action(user_id=self.fake_action_0.
                                       get('user_id'),
                                       action_id=action_id)

        self.assertEqual(len(result), 0)

    def test_add_and_update_action(self):
        action_doc = copy.deepcopy(self.fake_action_0)
        action_id = self.dbapi.add_action(user_id=self.fake_action_0.
                                          get('user_id'),
                                          doc=action_doc)
        self.assertIsNotNone(action_id)

        patch_doc = copy.deepcopy(self.fake_action_2)

        result = self.dbapi.update_action(user_id=self.fake_action_2.
                                          get('user_id'),
                                          patch_doc=patch_doc,
                                          action_id=action_id)

        self.assertIsNotNone(result)

        self.assertEqual(result, action_id)

        result = self.dbapi.get_action(user_id=self.fake_action_2.
                                       get('user_id'),
                                       action_id=action_id)

        self.assertEqual(result.get('max_retries'),
                         self.fake_action_2.get('max_retries'))

        self.assertEqual(result.get('max_retries_interval'),
                         self.fake_action_2.get('max_retries_interval'))

        freezer_action = result.get('freezer_action')

        self.assertEqual(freezer_action.get('action'),
                         self.freezer_action_2.get('action'))

    def test_add_and_replace_action(self):
        action_doc = copy.deepcopy(self.fake_action_0)
        action_id = self.dbapi.add_action(user_id=self.fake_action_0.
                                          get('user_id'),
                                          doc=action_doc)
        self.assertIsNotNone(action_id)

        patch_doc = copy.deepcopy(self.fake_action_2)

        result = self.dbapi.replace_action(user_id=self.fake_action_2.
                                           get('user_id'),
                                           doc=patch_doc,
                                           action_id=action_id)

        self.assertIsNotNone(result)

        self.assertEqual(result, action_id)
        result = self.dbapi.get_action(user_id=self.fake_action_2.
                                       get('user_id'),
                                       action_id=action_id)

        self.assertEqual(result.get('max_retries'),
                         self.fake_action_2.get('max_retries'))

        self.assertEqual(result.get('max_retries_interval'),
                         self.fake_action_2.get('max_retries_interval'))

        freezer_action = result.get('freezer_action')

        self.assertEqual(freezer_action.get('action'),
                         self.freezer_action_2.get('action'))
        patch_doc1 = copy.deepcopy(self.fake_action_0)
        result = self.dbapi.replace_action(user_id=self.fake_action_2.
                                           get('user_id'),
                                           doc=patch_doc1,
                                           action_id=self.fake_action_id)
        self.assertIsNotNone(result)
        result = self.dbapi.get_action(user_id=self.fake_action_2.
                                       get('user_id'),
                                       action_id=self.fake_action_id)
        self.assertEqual(result.get('action_id'), self.fake_action_id)

    def test_add_and_search_action(self):
        count = 0
        actionids = []
        while(count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)
            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=10,
                                          offset=0)

        self.assertIsNotNone(result)

        self.assertEqual(len(result), 10)

        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(actionids[index], actionmap['action_id'])

    def test_action_list_with_search_match_and_match_not(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
                if count in [4, 12]:
                    doc['freezer_action']['mode'] = 'nova'
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match_not': [{'mode': 'nova'}],
                      'match': [{'max_retries': 10}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 3)
        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(10, actionmap['max_retries'])
            self.assertEqual('fs',
                             actionmap['freezer_action']['mode'])

    def test_action_list_with_search_match_list(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
                if count in [4, 12]:
                    doc['freezer_action']['mode'] = 'nova'
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match': [{'max_retries': 10},
                                {'mode': 'nova'}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(10, actionmap['max_retries'])
            self.assertEqual('nova',
                             actionmap['freezer_action']['mode'])

    def test_action_list_with_search_match_not_list(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
                if count in [4, 12]:
                    doc['freezer_action']['mode'] = 'nova'
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match_not':
                      [{'mode': 'nova'},
                       {'max_retries': 5}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 3)
        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(10, actionmap['max_retries'])
            self.assertEqual('fs',
                             actionmap['freezer_action']['mode'])

    def test_action_list_with_search_with_all_opt_one_match(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match': [{'_all': '[{"max_retries": 10}]'}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 5)
        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(10, actionmap['max_retries'])

    def test_action_list_with_search_with_all_opt_two_matchs(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
                if count in [4, 12]:
                    doc['freezer_action']['mode'] = 'nova'
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match':
                      [{'_all':
                        '[{"max_retries": 10}, '
                        '{"mode": "nova"}]'}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 2)
        for index in range(len(result)):
            actionmap = result[index]
            self.assertEqual(10, actionmap['max_retries'])
            self.assertEqual('nova',
                             actionmap['freezer_action']['mode'])

    def test_action_list_with_search_with_error_all_opt_return_alltuples(self):
        count = 0
        actionids = []
        while (count < 20):
            doc = copy.deepcopy(self.fake_action_3)
            action_id = common.get_fake_action_id()
            doc['action_id'] = action_id
            if count in [0, 4, 8, 12, 16]:
                doc['max_retries'] = 10
            result = self.dbapi.add_action(user_id=self.fake_action_3.
                                           get('user_id'),
                                           doc=doc)

            self.assertIsNotNone(result)
            self.assertEqual(result, action_id)
            actionids.append(action_id)
            count += 1

        search_opt = {'match': [{'_all': '{"max_retries": 10}'}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 20)
        search_opt = {'match': [{'_all': 'max_retries=10'}]}
        result = self.dbapi.search_action(user_id=self.fake_action_3.
                                          get('user_id'),
                                          limit=20,
                                          offset=0,
                                          search=search_opt)

        self.assertIsNotNone(result)
        self.assertEqual(len(result), 20)

    @patch('freezer_api.db.sqlalchemy.api.get_action')
    def test_raise_add_action(self, mock_get_action):
        mock_get_action.return_value = mock.MagicMock()
        self.assertRaises(freezer_api_exc.DocumentExists,
                          self.dbapi.add_action, self.fake_user_id,
                          self.fake_action_0)



"""
Copyright 2015 Hewlett-Packard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import random

import falcon
from unittest import mock
from unittest.mock import patch

from freezer_api.api.v1 import sessions as v1_sessions
from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestSessionsCollectionResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestSessionsCollectionResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_session_0[
            'user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v1_sessions.SessionsCollectionResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_empty_list(self):
        self.mock_db.search_session.return_value = []
        expected_result = {'sessions': []}
        self.resource.on_get(self.mock_req, self.mock_req)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_get_return_correct_list(self):
        self.mock_db.search_session.return_value = [
            common.get_fake_session_0(), common.get_fake_session_1()]
        expected_result = {'sessions': [common.get_fake_session_0(),
                                        common.get_fake_session_1()]}
        self.resource.on_get(self.mock_req, self.mock_req)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_post_raises_when_missing_body(self):
        self.mock_db.add_session.return_value = common.fake_session_0[
            'session_id']
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req, self.mock_req)

    def test_on_post_inserts_correct_data(self):
        session = common.get_fake_session_0()
        self.mock_json_body.return_value = session
        self.mock_db.add_session.return_value = 'pjiofrdslaikfunr'
        expected_result = {'session_id': 'pjiofrdslaikfunr'}
        self.resource.on_post(self.mock_req, self.mock_req)
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestSessionsResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestSessionsResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_session_0[
            'user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v1_sessions.SessionsResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v1_sessions.SessionsResource)

    def test_on_get_return_no_result_and_404_when_not_found(self):
        self.mock_db.get_session.return_value = None
        self.mock_req.body = None
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_session_0['session_id'])
        self.assertIsNone(self.mock_req.body)
        self.assertEqual(falcon.HTTP_404, self.mock_req.status)

    def test_on_get_return_correct_data(self):
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_session_0['session_id'])
        result = self.mock_req.body
        self.assertEqual(common.get_fake_session_0(), result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_delete_removes_proper_data(self):
        self.resource.on_delete(self.mock_req, self.mock_req,
                                common.fake_session_0['session_id'])
        result = self.mock_req.body
        expected_result = {'session_id': common.fake_session_0['session_id']}
        self.assertEqual(falcon.HTTP_204, self.mock_req.status)
        self.assertEqual(expected_result, result)

    def test_on_patch_ok_with_some_fields(self):
        new_version = random.randint(0, 99)
        self.mock_db.update_session.return_value = new_version
        patch_doc = {'some_field': 'some_value',
                     'because': 'size_matters'}
        self.mock_json_body.return_value = patch_doc

        expected_result = {'session_id': common.fake_session_0['session_id'],
                           'version': new_version}

        self.resource.on_patch(self.mock_req, self.mock_req,
                               common.fake_session_0['session_id'])
        self.mock_db.update_session.assert_called_with(
            user_id=common.fake_session_0['user_id'],
            session_id=common.fake_session_0['session_id'],
            patch_doc=patch_doc)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)

    def test_on_post_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.replace_session.return_value = new_version
        session = common.get_fake_session_0()
        self.mock_json_body.return_value = session
        expected_result = {'session_id': common.fake_session_0['session_id'],
                           'version': new_version}

        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    def test_on_post_raises_when_db_replace_session_raises(self):
        self.mock_db.replace_session.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        session = common.get_fake_session_0()
        self.mock_json_body.return_value = session
        self.assertRaises(exceptions.AccessForbidden, self.resource.on_post,
                          self.mock_req,
                          self.mock_req,
                          common.fake_session_0['session_id'])


class TestSessionsAction(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestSessionsAction, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_session_0[
            'user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v1_sessions.SessionsAction(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v1_sessions.SessionsAction)

    def test_on_post_raises_when_unable_to_read_action_from_body(self):
        self.mock_json_body.return_value = {}
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req,
                          self.mock_req,
                          common.fake_session_0['session_id'])

    def test_on_post_start_action_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version

        action = {"start": {
            "job_id": 'job_id_2',
            "current_tag": 5
        }}
        self.mock_json_body.return_value = action
        expected_result = {'result': 'success',
                           'session_tag': 6}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    def test_on_post_start_action_raises_BadDataFormat_when_job_not_in_session(
            self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version
        action = {"start": {
            "job_id": 'missedme',
            "current_tag": 5
        }}
        self.mock_json_body.return_value = action
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req,
                          self.mock_req, common.fake_session_0['session_id'])

    def test_on_post_start_action_raises_BadDataFormat_when_curr_tag_too_high(
            self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version
        action = {"start": {
            "job_id": 'missedme',
            "current_tag": 6
        }}
        self.mock_json_body.return_value = action
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req,
                          self.mock_req, common.fake_session_0['session_id'])

    def test_on_post_end_action_ok(self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version
        action = {"end": {
            "job_id": 'job_id_2',
            "current_tag": 5,
            "result": "success"
        }}
        self.mock_json_body.return_value = action
        expected_result = {'result': 'success',
                           'session_tag': 5}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    def test_on_post_end_action_raises_BadDataFormat_when_job_not_in_session(
            self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version
        action = {"end": {
            "job_id": 'ahahahahah',
            "current_tag": 5,
            "result": "success"
        }}
        self.mock_json_body.return_value = action
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req,
                          self.mock_req, common.fake_session_0['session_id'])

    def test_on_post_raises_MethodNotImplemented_when_methon_not_implemented(
            self):
        new_version = random.randint(0, 99)
        self.mock_db.get_session.return_value = common.get_fake_session_0()
        self.mock_db.update_session.return_value = new_version
        action = {"method_not_implemented": {
            "job_id": 'ahahahahah',
            "current_tag": 5,
            "result": "success"
        }}
        self.mock_json_body.return_value = action
        self.assertRaises(exceptions.MethodNotImplemented,
                          self.resource.on_post, self.mock_req,
                          self.mock_req, common.fake_session_0['session_id'])

    @patch('freezer_api.api.v1.sessions.time')
    def test_on_post_start_succeeds_in_holdoff_if_tag_needs_not_increment(
            self, mock_time):
        mock_time.time.return_value = 1000
        new_version = random.randint(0, 99)
        session_doc = common.get_fake_session_0()
        session_doc['time_start'] = 999
        self.mock_db.get_session.return_value = session_doc
        self.mock_db.update_session.return_value = new_version
        action = {"start": {
            "job_id": 'job_id_2',
            "current_tag": 4
        }}
        self.mock_json_body.return_value = action
        expected_result = {'result': 'success',
                           'session_tag': 5}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    @patch('freezer_api.api.v1.sessions.time')
    def test_on_post_start_replies_holdoff_if_tag_would_increment(self,
                                                                  mock_time):
        mock_time.time.return_value = 1000
        new_version = random.randint(0, 99)
        session_doc = common.get_fake_session_0()
        session_doc['time_start'] = 999
        self.mock_db.get_session.return_value = session_doc
        self.mock_db.update_session.return_value = new_version
        action = {"start": {
            "job_id": 'job_id_2',
            "current_tag": 5
        }}
        self.mock_json_body.return_value = action
        expected_result = {'result': 'hold-off',
                           'session_tag': 5}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)

    @patch('freezer_api.api.v1.sessions.time')
    def test_on_post_start_outofholdoff_replies_outofsync_when_tag_too_low(
            self, mock_time):
        mock_time.time.return_value = 2000
        new_version = random.randint(0, 99)
        session_doc = common.get_fake_session_0()
        session_doc['time_start'] = 999
        self.mock_db.get_session.return_value = session_doc
        self.mock_db.update_session.return_value = new_version
        action = {"start": {
            "job_id": 'job_id_2',
            "current_tag": 2
        }}
        self.mock_json_body.return_value = action
        expected_result = {'result': 'out-of-sync',
                           'session_tag': 5}
        self.resource.on_post(self.mock_req, self.mock_req,
                              common.fake_session_0['session_id'])
        self.assertEqual(falcon.HTTP_202, self.mock_req.status)
        self.assertEqual(expected_result, self.mock_req.body)


class TestSessions(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestSessions, self).setUp()
        self.session_doc = {}
        self.session = v1_sessions.Session(self.session_doc)

    def test_create_resource(self):
        self.assertIsInstance(self.session, v1_sessions.Session)

    def test_overall_result_running(self):
        self.session_doc['jobs'] = {'job1': {'status': 'completed',
                                             'result': 'success'},
                                    'job2': {'status': 'running',
                                             'result': ''}}
        res = self.session.get_job_overall_result()
        self.assertEqual('running', res)

    def test_overall_result_fail(self):
        self.session_doc['jobs'] = {'job1': {'status': 'completed',
                                             'result': 'success'},
                                    'job2': {'status': 'completed',
                                             'result': 'fail'}}
        res = self.session.get_job_overall_result()
        self.assertEqual('fail', res)

    def test_overall_result_success(self):
        self.session_doc['jobs'] = {'job1': {'status': 'completed',
                                             'result': 'success'},
                                    'job2': {'status': 'completed',
                                             'result': 'success'}}
        res = self.session.get_job_overall_result()
        self.assertEqual('success', res)


class TestSessionsJobs(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestSessionsJobs, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = common.fake_session_0[
            'user_id']
        self.mock_req.status = falcon.HTTP_200
        self.resource = v1_sessions.SessionsJob(self.mock_db)

    def test_create_resource(self):
        self.assertIsInstance(self.resource, v1_sessions.SessionsJob)

    def test_on_put_adds_job_to_session_jobs(self):
        session = common.get_fake_session_0()
        job = common.get_fake_job_0()
        job_info = {job['job_id']: {'client_id': job['client_id'],
                                    'status': job['job_schedule']['status'],
                                    'result': job['job_schedule']['result'],
                                    'time_started': job['job_schedule'][
                                        'time_started'],
                                    'time_ended': job['job_schedule'][
                                        'time_ended']}}
        session_update_doc = {'jobs': job_info}

        self.mock_db.get_session.return_value = session
        self.mock_db.get_job.return_value = job

        self.resource.on_put(self.mock_req, self.mock_req,
                             session['session_id'],
                             job['job_id'])
        self.mock_db.update_session.assert_called_with(
            user_id=session['user_id'],
            session_id=session['session_id'],
            patch_doc=session_update_doc)

    def test_on_put_updates_job_with_session_info(self):
        session = common.get_fake_session_0()
        job = common.get_fake_job_0()

        self.mock_db.get_session.return_value = session
        self.mock_db.get_job.return_value = job

        job_update_doc = {
            'session_id': session['session_id'],
            'session_tag': session['session_tag'],
            'job_schedule': session['schedule']
        }

        self.resource.on_put(self.mock_req, self.mock_req,
                             session['session_id'],
                             job['job_id'])
        self.mock_db.update_job.assert_called_with(user_id=session['user_id'],
                                                   job_id=job['job_id'],
                                                   patch_doc=job_update_doc)

    def test_on_delete_removes_job_from_session_jobs(self):
        session = common.get_fake_session_0()
        updated_session = common.get_fake_session_1()
        job = common.get_fake_job_0()

        self.mock_db.get_session.return_value = session
        self.mock_db.get_job.return_value = job

        self.resource.on_delete(self.mock_req, self.mock_req,
                                session['session_id'],
                                'job_id_2')

        self.mock_db.replace_session.assert_called_with(
            user_id=session['user_id'],
            session_id=session['session_id'],
            doc=updated_session)

    def test_on_delete_removes_session_info_from_job_and_stops_job(self):
        session = common.get_fake_session_0()
        job = common.get_fake_job_0()

        self.mock_db.get_session.return_value = session
        self.mock_db.get_job.return_value = job

        job_update_doc = {
            'session_id': '',
            'session_tag': 0,
            'job_schedule': {
                'event': 'stop'
            }
        }

        self.resource.on_delete(self.mock_req, self.mock_req,
                                session['session_id'],
                                job['job_id'])

        self.mock_db.update_job.assert_called_with(user_id=session['user_id'],
                                                   job_id=job['job_id'],
                                                   patch_doc=job_update_doc)



"""
(c) Copyright 2016 Hewlett-Packard Enterprise Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from oslo_log import log
from oslo_serialization import jsonutils
from oslo_utils import encodeutils

from freezer_api import context

LOG = log.getLogger(__name__)


def inject_context(req, resp, params):
    user_id = req.get_header('X-USER-ID')
    request_id = req.get_header('X-Openstack-Request-ID')
    auth_token = req.get_header('X-AUTH-TOKEN')
    tenant = req.get_header('X-TENANT-ID')

    roles = req.get_header('X-ROLES')
    roles = roles and roles.split(',') or []

    ctxt = context.FreezerContext(auth_token=auth_token,
                                  user=user_id,
                                  tenant=tenant,
                                  request_id=request_id,
                                  roles=roles)

    req.env['freezer.context'] = ctxt
    LOG.info('Request context set')


def before_hooks():
    return [inject_context]


def json_encode(obj):
    return encodeutils.safe_encode(jsonutils.dumps(obj), 'utf-8')


def json_decode(binary):
    return jsonutils.loads(binary, 'utf-8')


class FuncMiddleware(object):
    """
    Injecting some function as middleware for freezer-api
    """
    def __init__(self, func):
        self.func = func

    def process_resource(self, req, resp, resource, params=None):
        return self.func(req, resp, params)



# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import falcon
from unittest import mock

from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestExceptions(unittest.TestCase):

    def setUp(self):
        self.ex = mock.Mock()
        self.ex.message = 'test exception'
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.context = {}

    def test_FreezerAPIException(self):
        e = exceptions.FreezerAPIException(message='testing')
        self.assertRaises(falcon.HTTPError,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)

    def test_BadDataFormat(self):
        e = exceptions.BadDataFormat(message='testing')
        self.assertRaises(falcon.HTTPBadRequest,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)

    def test_DocumentExists(self):
        e = exceptions.DocumentExists(message='testing')
        self.assertRaises(falcon.HTTPConflict,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)

    def test_StorageEngineError(self):
        e = exceptions.StorageEngineError(message='testing')
        self.assertRaises(falcon.HTTPInternalServerError,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)

    def test_DocumentNotFound(self):
        e = exceptions.DocumentNotFound(message='testing')
        self.assertRaises(falcon.HTTPError,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)

    def test_AccessForbidden(self):
        e = exceptions.AccessForbidden(message='testing')
        self.assertRaises(falcon.HTTPForbidden,
                          e.handle, self.ex, self.mock_req, self.mock_req,
                          None)



"""
(C) Copyright 2016-2018 Hewlett Packard Enterprise Development Company LP
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from oslo_config import cfg
from oslo_log import log
from oslo_utils import importutils

CONF = cfg.CONF
LOG = log.getLogger(__name__)

# storage backend options to be registered
_OPTS = [
    cfg.StrOpt("backend",
               help="Database backend section name. This section "
                    "will be loaded by the proper driver to connect to "
                    "the database."
               ),
    cfg.StrOpt('driver',
               default='freezer_api.storage.elastic.ElasticSearchEngine',
               help="Database driver to be used."
               )
]


def get_db(driver=None):
    """Automatically loads the database driver to be used."""
    storage = CONF.get('storage')
    if not driver:
        driver = storage['driver']
    driver_instance = importutils.import_object(
        driver,
        backend=storage['backend']
    )

    return driver_instance



# (c) Copyright 2018 ZTE Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Freezer-api DB test base class."""

import fixtures
from oslo_config import cfg
from oslo_db import options as db_options

from freezer_api.db.sqlalchemy import api as sqla_api
from freezer_api.db.sqlalchemy import models

from freezer_api.db import manager

from freezer_api.tests.unit import common

CONF = cfg.CONF

_DB_CACHE = None

IN_MEM_DB_CONN_STRING = 'sqlite://'


class Database(fixtures.Fixture):

    def __init__(self, db_api):
        self.initialize_sql_session(IN_MEM_DB_CONN_STRING)
        self.engine = sqla_api.get_engine()
        self.engine.dispose()
        conn = self.engine.connect()
        self.setup_sqlite()

        self._DB = "".join(line for line in conn.connection.iterdump())
        self.engine.dispose()

    def initialize_sql_session(self, connection_str):
        db_options.set_defaults(
            CONF,
            connection=connection_str)
        CONF.set_override('connection', IN_MEM_DB_CONN_STRING,
                          group='database')

    def setup_sqlite(self):
        models.register_models(self.engine)

    def setUp(self):
        super(Database, self).setUp()

        conn = self.engine.connect()
        conn.connection.executescript(self._DB)
        self.addCleanup(self.engine.dispose)


class DbTestCase(common.FreezerBaseTestCase):

    def setUp(self):
        dbbackend = "sqlalchemy"
        dbdriver = "sqlalchemy"
        super(DbTestCase, self).setUp()
        db_driver = manager.get_db_driver(dbdriver,
                                          backend=dbbackend)
        self.dbapi = db_driver.IMPL
        global _DB_CACHE
        if not _DB_CACHE:
            _DB_CACHE = Database(sqla_api)
        self.useFixture(_DB_CACHE)



# -*- encoding: utf-8 -*-
# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

import elasticsearch
from unittest import mock
from unittest.mock import patch

from oslo_config import cfg

from freezer_api.common import exceptions
from freezer_api.db.elasticsearch.driver import ElasticSearchDB
from freezer_api.storage import elasticv2 as elastic
from freezer_api.tests.unit import common

CONF = cfg.CONF


class TypeManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.type_manager = elastic.TypeManagerV2(self.mock_es,
                                                  'base_doc_type',
                                                  'freezer')

    def test_get_base_search_filter(self):
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        q = self.type_manager.get_base_search_filter(project_id='tecs',
                                                     user_id='my_user_id',
                                                     search=my_search)
        expected_q = [
            {
                'term': {
                    'project_id': 'tecs'
                }
            },
            {
                'term': {
                    'user_id': 'my_user_id'
                }
            },
            {
                'query': {
                    'bool': {
                        'must_not': [],
                        'must': [
                            {
                                'match': {
                                    'some_field': 'some text'
                                }
                            },
                            {
                                'match': {
                                    'description': 'some other text'
                                }
                            }
                        ]
                    }
                }
            }
        ]
        self.assertEqual(expected_q, q)

    def test_get_ok(self):
        self.mock_es.get.return_value = common.fake_job_0_elasticsearch_found
        res = self.type_manager.get(project_id='tecs',
                                    user_id=common.fake_job_0_user_id,
                                    doc_id=common.fake_job_0_job_id)
        self.assertEqual(common.fake_job_0, res)

    def test_get_raise_DocumentNotFound_when_doc_not_found(self):
        self.mock_es.get.side_effect = elasticsearch.TransportError(
            'regular test failure')
        self.assertRaises(exceptions.DocumentNotFound, self.type_manager.get,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          doc_id=common.fake_job_0_job_id)

    def test_get_raise_StorageEngineError_when_db_raises(self):
        self.mock_es.get.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError, self.type_manager.get,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          doc_id=common.fake_job_0_job_id)

    def test_raise_get_search_query(self):
        my_search_error = ['a', 'b']
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.get_search_query,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          doc_id=common.fake_job_0_job_id,
                          search=my_search_error)

    def test_get_raises_AccessForbidden_when_project_id_not_match(self):
        self.mock_es.get.return_value = common.fake_job_0_elasticsearch_found
        self.assertRaises(exceptions.AccessForbidden, self.type_manager.get,
                          project_id='tecs1',
                          doc_id=common.fake_job_0_job_id)

    def test_get_raises_AccessForbidden_when_user_id_not_match(self):
        self.mock_es.get.return_value = common.fake_job_0_elasticsearch_found
        self.assertRaises(exceptions.AccessForbidden, self.type_manager.get,
                          project_id='tecs',
                          user_id='obluraschi',
                          doc_id=common.fake_job_0_job_id)

    def test_search_ok(self):
        self.mock_es.search.return_value = common.fake_data_0_elasticsearch_hit
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'some_field':
                                                            'some text'
                                                    }
                                                },
                                                {
                                                    'match': {
                                                        'description':
                                                            'some other text'
                                                    }
                                                }]}}}
                            ]}}}}}
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.type_manager.search(project_id='tecs',
                                       user_id='my_user_id',
                                       doc_id='mydocid',
                                       search=my_search, offset=7, limit=19)
        self.mock_es.search.assert_called_with(index='freezer',
                                               doc_type='base_doc_type',
                                               size=19, from_=7,
                                               body=expected_q)
        self.assertEqual([common.fake_data_0_backup_metadata], res)

    def test_search_raise_StorageEngineError_when_search_raises(self):
        self.mock_es.search.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.search, project_id='tecs',
                          user_id='my_user_id', doc_id='mydocid')

    @patch('freezer_api.storage.elasticv2.elasticsearch.Elasticsearch')
    def test_search_raise_StorageEngineError_when_ConnectionError(self,
                                                                  mock_es):
        self.mock_es.search.side_effect = elasticsearch.ConnectionError(
            'regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.search, project_id='tecs',
                          user_id='my_user_id', doc_id='mydocid')

    def test_insert_ok(self):
        self.mock_es.index.return_value = {'created': True, '_version': 15}
        test_doc = {'test_key_412': 'test_value_412', '_version': 5}
        res = self.type_manager.insert(doc=test_doc)
        self.assertEqual((True, 15), res)
        self.mock_es.index.assert_called_with(index='freezer',
                                              doc_type='base_doc_type',
                                              body=test_doc, id=None)

    def test_insert_raise_StorageEngineError_on_ES_Exception(self):
        self.mock_es.index.side_effect = Exception('regular test failure')
        test_doc = {'test_key_412': 'test_value_412', '_version': 5}
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.insert, doc=test_doc)
        self.mock_es.index.assert_called_with(index='freezer',
                                              doc_type='base_doc_type',
                                              body=test_doc, id=None)

    def test_insert_raise_StorageEngineError_on_ES_TransportError_exception(
            self):
        self.mock_es.index.side_effect = elasticsearch.TransportError(
            500, 'regular test failure'
        )
        test_doc = {'test_key_412': 'test_value_412', '_version': 5}
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.insert, doc=test_doc)
        self.mock_es.index.assert_called_with(index='freezer',
                                              doc_type='base_doc_type',
                                              body=test_doc, id=None)

    def test_insert_raise_DocumentExists_on_ES_TransportError409_exception(
            self):
        self.mock_es.index.side_effect = elasticsearch.TransportError(
            409, 'regular test failure'
        )
        test_doc = {'test_key_412': 'test_value_412', '_version': 5}
        self.assertRaises(exceptions.DocumentExists, self.type_manager.insert,
                          doc=test_doc)
        self.mock_es.index.assert_called_with(index='freezer',
                                              doc_type='base_doc_type',
                                              body=test_doc, id=None)

    @patch('freezer_api.storage.elasticv2.elasticsearch.Elasticsearch')
    def test_delete_raises_StorageEngineError_on_scan_exception(
            self, mock_elasticsearch):
        doc_id = 'mydocid345'
        mock_elasticsearch.search.side_effect = Exception(
            'regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.delete, project_id='tecs',
                          user_id='my_user_id', doc_id=doc_id)

    @patch('freezer_api.storage.elasticv2.elasticsearch.Elasticsearch')
    def test_delete_raises_StorageEngineError_on_delete_exception(
            self, mock_elasticsearch):
        doc_id = 'mydocid345'
        mock_elasticsearch.search.return_value = [
            {'_id': 'cicciopassamilolio'}]
        self.mock_es.delete.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.delete, project_id='tecs',
                          user_id='my_user_id', doc_id=doc_id)

    def test_delete_raises_StorageEngineError_on_es_delete(
            self):
        results = {'hits': {'hits': [{'_id': mock.Mock}]}}
        self.mock_es.search.return_value = results
        doc_id = 'mydocid345'
        self.mock_es.delete.side_effect = Exception(
            'regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.type_manager.delete, project_id='tecs',
                          user_id='my_user_id', doc_id=doc_id)

    def test_delete_return_none_when_nothing_is_deleted(self):
        doc_id = 'mydocid345'
        ret_data = {"hits": {"hits": []}}
        self.mock_es.search.return_value = ret_data
        res = self.type_manager.delete(project_id='tecs',
                                       user_id='my_user_id', doc_id=doc_id)
        self.assertIsNone(res, 'invalid res {0}'.format(res))

    def test_delete_return_correct_id_on_success(self):
        doc_id = 'mydocid345'
        ret_data = {"hits": {"hits": [{
            "_id": "cicciopassamilolio"
        }]}}
        self.mock_es.search.return_value = ret_data
        res = self.type_manager.delete(project_id='tecs',
                                       user_id='my_user_id', doc_id=doc_id)
        self.assertEqual(
            'cicciopassamilolio', res, 'invalid res {0}'.format(res)
        )


class TestBackupManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.backup_manager = elastic.BackupTypeManagerV2(self.mock_es,
                                                          'backups')

    def test_get_search_query(self):
        my_search = {'match': [{'backup_name': 'my_backup'}, {'mode': 'fs'}],
                     "time_before": 1428510506,
                     "time_after": 1428510506
                     }
        q = self.backup_manager.get_search_query(project_id='tecs',
                                                 user_id='my_user_id',
                                                 doc_id='my_doc_id',
                                                 search=my_search)
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'backup_name':
                                                            'my_backup'
                                                    }
                                                },
                                                {
                                                    'match': {
                                                        'mode': 'fs'
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    'term': {
                                        'backup_id': 'my_doc_id'
                                    }
                                },
                                {
                                    'range': {
                                        'timestamp': {
                                            'gte': 1428510506
                                        }
                                    }
                                },
                                {
                                    'range': {
                                        'timestamp': {
                                            'lte': 1428510506
                                        }
                                    }
                                }
                            ]
                        }
                    }
                }
            }
        }
        self.assertEqual(expected_q, q)


class ClientTypeManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.client_manager = elastic.ClientTypeManagerV2(self.mock_es,
                                                          'clients')

    def test_get_search_query(self):
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        q = self.client_manager.get_search_query(project_id='tecs',
                                                 user_id='my_user_id',
                                                 doc_id='my_doc_id',
                                                 search=my_search)
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'some_field':
                                                            'some text'
                                                    }
                                                },
                                                {
                                                    'match': {
                                                        'description':
                                                            'some other text'
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    'term': {
                                        'client.client_id': 'my_doc_id'
                                    }
                                }
                            ]
                        }
                    }
                }
            }
        }
        self.assertEqual(expected_q, q)


class JobTypeManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.job_manager = elastic.JobTypeManagerV2(self.mock_es, 'clients')

    def test_get_search_query(self):
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        q = self.job_manager.get_search_query(project_id='tecs',
                                              user_id='my_user_id',
                                              doc_id='my_doc_id',
                                              search=my_search)
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'some_field':
                                                            'some text'
                                                    }
                                                },
                                                {
                                                    'match': {
                                                        'description':
                                                            'some other text'
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {'term': {
                                    'job_id': 'my_doc_id'
                                }
                                }
                            ]}}}}}
        self.assertEqual(expected_q, q)

    def test_update_ok(self):
        self.mock_es.update.return_value = {
            u'_id': u'd6c1e00d-b9c1-4eb3-8219-1e83c02af101',
            u'_index': u'freezer',
            u'_type': u'jobs',
            u'_version': 3
        }
        res = self.job_manager.update(job_id=common.fake_job_0_job_id,
                                      job_update_doc={'status': 'sleepy'})
        self.assertEqual(3, res)
        self.mock_es.update.assert_called_with(
            index=self.job_manager.index,
            doc_type=self.job_manager.doc_type,
            id=common.fake_job_0_job_id,
            body={"doc": {'status': 'sleepy'}}
        )

    def test_update_raise_DocumentNotFound_when_not_found(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            'regular test failure', 1)
        self.assertRaises(exceptions.DocumentNotFound, self.job_manager.update,
                          job_id=common.fake_job_0_job_id,
                          job_update_doc={'status': 'sleepy'})

    def test_update_raise_DocumentExists_when_elasticsearch_returns_409(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            409, 'regular test failure'
        )
        self.assertRaises(exceptions.DocumentExists, self.job_manager.update,
                          job_id=common.fake_job_0_job_id,
                          job_update_doc={'status': 'sleepy'})

    def test_update_raise_StorageEngineError_when_db_raises(self):
        self.mock_es.update.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.job_manager.update,
                          job_id=common.fake_job_0_job_id,
                          job_update_doc={'status': 'sleepy'})


class ActionTypeManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.action_manager = elastic.ActionTypeManagerV2(self.mock_es,
                                                          'actions')

    def test_get_search_query(self):
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        q = self.action_manager.get_search_query(project_id='tecs',
                                                 user_id='my_user_id',
                                                 doc_id='my_doc_id',
                                                 search=my_search)
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'some_field':
                                                            'some text'}
                                                },
                                                {
                                                    'match': {
                                                        'description':
                                                            'some other text'}
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    'term': {
                                        'action_id': 'my_doc_id'
                                    }
                                }
                            ]}}}}}
        self.assertEqual(expected_q, q)

    def test_update_ok(self):
        self.mock_es.update.return_value = {
            u'_id': u'd6c1e00d-b9c1-4eb3-8219-1e83c02af101',
            u'_index': u'freezer',
            u'_type': u'actions',
            u'_version': 3
        }
        res = self.action_manager.update(action_id='poiuuiop7890',
                                         action_update_doc={
                                             'status': 'sleepy'})
        self.assertEqual(3, res)
        self.mock_es.update.assert_called_with(
            index=self.action_manager.index,
            doc_type=self.action_manager.doc_type,
            id='poiuuiop7890',
            body={"doc": {'status': 'sleepy'}}
        )

    def test_update_raise_DocumentNotFound_when_not_found(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            'regular test failure', 1)
        self.assertRaises(exceptions.DocumentNotFound,
                          self.action_manager.update,
                          action_id='asdfsadf',
                          action_update_doc={'status': 'sleepy'})

    def test_update_raise_DocumentExists_when_elasticsearch_returns_409(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            409, 'regular test failure'
        )
        self.assertRaises(exceptions.DocumentExists,
                          self.action_manager.update,
                          action_id='pepepepepe2321',
                          action_update_doc={'status': 'sleepy'})

    def test_update_raise_StorageEngineError_when_db_raises(self):
        self.mock_es.update.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.action_manager.update,
                          action_id='pepepepepe2321',
                          action_update_doc={'status': 'sleepy'})


class SessionTypeManagerV2(unittest.TestCase):
    def setUp(self):
        self.mock_es = mock.Mock()
        self.session_manager = elastic.SessionTypeManagerV2(self.mock_es,
                                                            'sessions')

    def test_get_search_query(self):
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        q = self.session_manager.get_search_query(project_id='tecs',
                                                  user_id='my_user_id',
                                                  doc_id='my_doc_id',
                                                  search=my_search)
        expected_q = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': [
                                {
                                    'term': {
                                        'project_id': 'tecs'
                                    }
                                },
                                {
                                    'term': {
                                        'user_id': 'my_user_id'
                                    }
                                },
                                {
                                    'query': {
                                        'bool': {
                                            'must_not': [],
                                            'must': [
                                                {
                                                    'match': {
                                                        'some_field':
                                                            'some text'
                                                    }
                                                },
                                                {
                                                    'match': {
                                                        'description':
                                                            'some other text'
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    'term': {
                                        'session_id': 'my_doc_id'
                                    }
                                }
                            ]}}}}}
        self.assertEqual(expected_q, q)

    def test_update_ok(self):
        self.mock_es.update.return_value = {
            u'_id': u'd6c1e00d-b9c1-4eb3-8219-1e83c02af101',
            u'_index': u'freezer',
            u'_type': u'actions',
            u'_version': 3
        }
        res = self.session_manager.update(session_id='poiuuiop7890',
                                          session_update_doc={
                                              'status': 'sleepy'})
        self.assertEqual(3, res)
        self.mock_es.update.assert_called_with(
            index=self.session_manager.index,
            doc_type=self.session_manager.doc_type,
            id='poiuuiop7890',
            body={"doc": {'status': 'sleepy'}})

    def test_update_raise_DocumentNotFound_when_not_found(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            'regular test failure', 1)
        self.assertRaises(exceptions.DocumentNotFound,
                          self.session_manager.update,
                          session_id='asdfsadf',
                          session_update_doc={'status': 'sleepy'})

    def test_update_raise_DocumentExists_when_elasticsearch_returns_409(self):
        self.mock_es.update.side_effect = elasticsearch.TransportError(
            409, 'regular test failure'
        )
        self.assertRaises(exceptions.DocumentExists,
                          self.session_manager.update,
                          session_id='pepepepepe2321',
                          session_update_doc={'status': 'sleepy'})

    def test_update_raise_StorageEngineError_when_db_raises(self):
        self.mock_es.update.side_effect = Exception('regular test failure')
        self.assertRaises(exceptions.StorageEngineError,
                          self.session_manager.update,
                          session_id='pepepepepe2321',
                          session_update_doc={'status': 'sleepy'})


class TestElasticSearchEngineV2_backup(unittest.TestCase, ElasticSearchDB):

    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_logging, mock_elasticsearch):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)
        self.eng.backup_manager = mock.Mock()

    def test_get_backup_userid_and_backup_id_return_ok(self):
        self.eng.backup_manager.get.return_value = (
            common.fake_data_0_wrapped_backup_metadata
        )
        res = self.eng.get_backup(project_id='tecs',
                                  user_id=common.fake_data_0_user_id,
                                  backup_id=common.fake_data_0_backup_id)

        self.assertEqual(common.fake_data_0_wrapped_backup_metadata, res)
        self.eng.backup_manager.get.assert_called_with(
            project_id=common.fake_data_0_wrapped_backup_metadata
            ['project_id'],
            user_id=common.fake_data_0_wrapped_backup_metadata['user_id'],
            doc_id=common.fake_data_0_wrapped_backup_metadata['backup_id']
        )

    def test_get_backup_list_with_userid_and_search_return_list(self):
        self.eng.backup_manager.search.return_value = [
            common.fake_data_0_wrapped_backup_metadata,
            common.fake_data_1_wrapped_backup_metadata]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_backup(project_id='tecs',
                                     user_id=common.fake_data_0_user_id,
                                     offset=3, limit=7,
                                     search=my_search)
        self.assertEqual(
            [
                common.fake_data_0_wrapped_backup_metadata,
                common.fake_data_1_wrapped_backup_metadata
            ], res
        )
        self.eng.backup_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_data_0_wrapped_backup_metadata['user_id'],
            search=my_search,
            limit=7, offset=3)

    def test_get_backup_list_with_userid_and_search_return_empty(self):
        self.eng.backup_manager.search.return_value = []
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_backup(project_id='tecs',
                                     user_id=common.fake_data_0_user_id,
                                     offset=3, limit=7,
                                     search=my_search)
        self.assertEqual([], res)
        self.eng.backup_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_data_0_wrapped_backup_metadata['user_id'],
            search=my_search,
            limit=7, offset=3)

    def test_get_backup_userid_and_backup_id_not_found_returns_empty(self):
        self.eng.backup_manager.get.return_value = None
        res = self.eng.get_backup(project_id='tecs',
                                  user_id=common.fake_data_0_user_id,
                                  backup_id=common.fake_data_0_backup_id)
        self.assertIsNone(res)
        self.eng.backup_manager.get.assert_called_with(
            project_id=common.fake_data_0_wrapped_backup_metadata
            ['project_id'],
            user_id=common.fake_data_0_wrapped_backup_metadata['user_id'],
            doc_id=common.fake_data_0_wrapped_backup_metadata['backup_id']
        )

    def test_add_backup_raises_when_data_is_malformed(self):
        self.assertRaises(exceptions.BadDataFormat, self.eng.add_backup,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          user_name=common.fake_data_0_user_name,
                          doc=common.fake_malformed_data_0_backup_metadata)

    def test_add_backup_ok(self):
        self.eng.backup_manager.search.return_value = []
        res = self.eng.add_backup(project_id='tecs',
                                  user_id=common.fake_data_0_user_id,
                                  user_name=common.fake_data_0_user_name,
                                  doc=common.fake_data_0_backup_metadata)
        self.assertTrue(res)

    def test_add_backup_raises_when_manager_insert_raises(self):
        self.eng.backup_manager.search.return_value = []
        self.eng.backup_manager.insert.side_effect = (
            exceptions.StorageEngineError('regular test failure')
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.add_backup,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          user_name=common.fake_data_0_user_name,
                          doc=common.fake_data_0_backup_metadata)

    def test_delete_backup_ok(self):
        self.eng.backup_manager.delete.return_value = (
            common.fake_data_0_backup_id
        )
        res = self.eng.delete_backup(project_id='tecs',
                                     user_id=common.fake_data_0_user_id,
                                     backup_id=common.fake_data_0_backup_id)
        self.assertEqual(common.fake_data_0_backup_id, res)

    def test_delete_backup_raises_when_es_delete_raises(self):
        self.eng.backup_manager.delete.side_effect = (
            exceptions.StorageEngineError()
        )
        self.assertRaises(exceptions.StorageEngineError,
                          self.eng.delete_backup,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          backup_id=common.fake_data_0_backup_id)


class TestElasticSearchEngine_client(unittest.TestCase, ElasticSearchDB):
    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_logging, mock_elasticsearch):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)
        self.eng.client_manager = mock.Mock()

    def test_get_client_userid_and_client_id_return_1elem_list_(self):
        self.eng.client_manager.search.return_value = [
            common.fake_client_entry_0]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.get_client(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            client_id=common.fake_client_info_0['client_id'],
            offset=6, limit=15,
            search=my_search)
        self.assertEqual([common.fake_client_entry_0], res)
        self.eng.client_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            doc_id=common.fake_client_info_0['client_id'],
            search=my_search,
            limit=15, offset=6)

    def test_get_client_list_with_userid_and_search_return_list(self):
        self.eng.client_manager.search.return_value = [
            common.fake_client_entry_0, common.fake_client_entry_1]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.get_client(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            offset=6, limit=15,
            search=my_search)
        self.assertEqual(
            [
                common.fake_client_entry_0,
                common.fake_client_entry_1
            ], res)
        self.eng.client_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            doc_id=None,
            search=my_search,
            limit=15, offset=6)

    def test_get_client_list_with_userid_and_search_return_empty_list(self):
        self.eng.client_manager.search.return_value = []
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.get_client(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            offset=6, limit=15,
            search=my_search)
        self.assertEqual([], res)
        self.eng.client_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_client_entry_0['user_id'],
            doc_id=None,
            search=my_search,
            limit=15, offset=6)

    def test_add_client_raises_when_data_is_malformed(self):
        doc = common.fake_client_info_0.copy()
        doc.pop('client_id')
        self.assertRaises(exceptions.BadDataFormat, self.eng.add_client,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_name,
                          doc=doc)

    def test_add_client_raises_when_doc_exists(self):
        self.eng.client_manager.search.return_value = [
            common.fake_client_entry_0]
        self.assertRaises(exceptions.DocumentExists, self.eng.add_client,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          doc=common.fake_client_info_0)

    def test_add_client_ok(self):
        self.eng.client_manager.search.return_value = []
        res = self.eng.add_client(project_id='tecs',
                                  user_id=common.fake_data_0_user_id,
                                  doc=common.fake_client_info_0)
        self.assertEqual(common.fake_client_info_0['client_id'], res)
        self.eng.client_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_data_0_user_id,
            doc_id=common.fake_client_info_0['client_id'])

    def test_add_client_raises_when_manager_insert_raises(self):
        self.eng.client_manager.search.return_value = []
        self.eng.client_manager.insert.side_effect = (
            exceptions.StorageEngineError('regular test failure')
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.add_client,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          doc=common.fake_client_info_0)

    def test_delete_client_ok(self):
        self.eng.client_manager.delete.return_value = (
            common.fake_client_info_0['client_id']
        )
        res = self.eng.delete_client(project_id='tecs',
                                     user_id=common.fake_data_0_user_id,
                                     client_id=common.fake_client_info_0[
                                         'client_id'])
        self.assertEqual(common.fake_client_info_0['client_id'], res)

    def test_delete_client_raises_when_es_delete_raises(self):
        self.eng.client_manager.delete.side_effect = (
            exceptions.StorageEngineError()
        )
        self.assertRaises(exceptions.StorageEngineError,
                          self.eng.delete_client,
                          project_id='tecs',
                          user_id=common.fake_data_0_user_id,
                          client_id=common.fake_client_info_0['client_id'])


class TestElasticSearchEngine_job(unittest.TestCase, ElasticSearchDB):
    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_elasticsearch, mock_logging):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)
        self.eng.job_manager = mock.Mock()

    def test_get_job_userid_and_job_id_return_doc(self):
        self.eng.job_manager.get.return_value = common.get_fake_job_0()
        res = self.eng.get_job(project_id='tecs',
                               user_id=common.fake_job_0['user_id'],
                               job_id=common.fake_job_0['job_id'])
        self.assertEqual(common.fake_job_0, res)
        self.eng.job_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_job_0['user_id'],
            doc_id=common.fake_job_0['job_id'])

    def test_get_job_userid_and_job_id_return_none(self):
        self.eng.job_manager.get.return_value = None
        res = self.eng.get_job(project_id='tecs',
                               user_id=common.fake_job_0['user_id'],
                               job_id=common.fake_job_0['job_id'])
        self.assertIsNone(res)
        self.eng.job_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_job_0['user_id'],
            doc_id=common.fake_job_0['job_id'])

    def test_get_job_with_userid_and_search_return_list(self):
        self.eng.job_manager.search.return_value = \
            [common.fake_job_0, common.fake_job_0]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_job(project_id='tecs',
                                  user_id=common.fake_job_0['user_id'],
                                  offset=6, limit=15,
                                  search=my_search)
        self.assertEqual([common.fake_job_0, common.fake_job_0], res)
        self.eng.job_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_job_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    def test_get_job_with_userid_and_search_return_empty_list(self):
        self.eng.job_manager.search.return_value = []
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_job(project_id='tecs',
                                  user_id=common.fake_job_0['user_id'],
                                  offset=6, limit=15,
                                  search=my_search)
        self.assertEqual([], res)
        self.eng.job_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_job_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    @patch('freezer_api.common.elasticv2_utils.JobDoc')
    def test_add_job_ok(self, mock_jobdoc):
        mock_jobdoc.create.return_value = common.get_fake_job_0()
        self.eng.job_manager.insert.return_value = (True, 1)
        res = self.eng.add_job(project_id='tecs',
                               user_id=common.fake_job_0_user_id,
                               doc=common.get_fake_job_0())
        self.assertEqual(common.fake_job_0_job_id, res)
        self.eng.job_manager.insert.assert_called_with(
            common.fake_job_0, common.fake_job_0_job_id
        )

    def test_add_job_raises_StorageEngineError_when_manager_insert_raises(
            self):
        self.eng.job_manager.get.return_value = None
        self.eng.job_manager.insert.side_effect = (
            exceptions.StorageEngineError('regular test failure')
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.add_job,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          doc=common.get_fake_job_0())

    def test_delete_job_ok(self):
        self.eng.job_manager.delete.return_value = common.fake_job_0['job_id']
        res = self.eng.delete_job(project_id='tecs',
                                  user_id=common.fake_job_0_user_id,
                                  job_id=common.fake_job_0_job_id)
        self.assertEqual(common.fake_job_0_job_id, res)

    def test_delete_client_raises_StorageEngineError_when_es_delete_raises(
            self):
        self.eng.job_manager.delete.side_effect = (
            exceptions.StorageEngineError()
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.delete_job,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          job_id=common.fake_job_0_job_id)

    def test_update_job_raises_DocumentNotFound_when_doc_not_exists(self):
        self.eng.job_manager.get.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        patch = {'job_id': 'black_milk'}
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_job,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          job_id=common.fake_job_0_job_id,
                          patch_doc=patch)

    def test_update_job_raises_DocumentNotFound_when_update_raises(
            self):
        self.eng.job_manager.get.return_value = common.get_fake_job_0()
        patch = {'job_id': 'black_milk'}
        self.eng.job_manager.update.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_job,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          job_id=common.fake_job_0_job_id,
                          patch_doc=patch)

    def test_update_job_returns_new_doc_version(self):
        self.eng.job_manager.get.return_value = common.get_fake_job_0()
        patch = {'job_id': 'group_four'}
        self.eng.job_manager.update.return_value = 11
        res = self.eng.update_job(project_id='tecs',
                                  user_id=common.fake_job_0_user_id,
                                  job_id=common.fake_job_0_job_id,
                                  patch_doc=patch)
        self.assertEqual(11, res)

    def test_replace_job_raises_AccessForbidden_when_job_manager_raises(
            self):
        self.eng.job_manager.get.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        self.eng.job_manager.insert.return_value = (True, 3)
        self.assertRaises(exceptions.AccessForbidden, self.eng.replace_job,
                          project_id='tecs',
                          user_id=common.fake_job_0_user_id,
                          job_id=common.fake_job_0_job_id,
                          doc=common.get_fake_job_0())

    def test_replace_job_returns_ok_when_doc_is_new(self):
        self.eng.job_manager.get.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        self.eng.job_manager.insert.return_value = (True, 1)
        res = self.eng.replace_job(project_id='tecs',
                                   user_id=common.fake_job_0_user_id,
                                   job_id=common.fake_job_0_job_id,
                                   doc=common.get_fake_job_0())
        self.assertEqual(1, res)

    def test_replace_job_returns_version_1_when_doc_is_overwritten(self):
        self.eng.job_manager.get.return_value = common.get_fake_job_0()
        self.eng.job_manager.insert.return_value = (False, 3)
        res = self.eng.replace_job(project_id='tecs',
                                   user_id=common.fake_job_0_user_id,
                                   job_id=common.fake_job_0_job_id,
                                   doc=common.get_fake_job_0())
        self.assertEqual(3, res)


class TestElasticSearchEngine_action(unittest.TestCase, ElasticSearchDB):
    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_elasticsearch, mock_logging):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)
        self.eng.action_manager = mock.Mock()

    def test_get_action_userid_and_action_id_return_doc(self):
        self.eng.action_manager.get.return_value = common.get_fake_action_0()
        res = self.eng.get_action(project_id='tecs',
                                  user_id=common.fake_action_0['user_id'],
                                  action_id=common.fake_action_0['action_id'])
        self.assertEqual(common.fake_action_0, res)
        self.eng.action_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_action_0['user_id'],
            doc_id=common.fake_action_0['action_id'])

    def test_get_action_userid_and_action_id_return_none(self):
        self.eng.action_manager.get.return_value = None
        res = self.eng.get_action(project_id='tecs',
                                  user_id=common.fake_action_0['user_id'],
                                  action_id=common.fake_action_0['action_id'])
        self.assertIsNone(res)
        self.eng.action_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_action_0['user_id'],
            doc_id=common.fake_action_0['action_id'])

    def test_get_action_with_userid_and_search_return_list(self):
        self.eng.action_manager.search.return_value = \
            [common.fake_action_0, common.fake_action_0]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_action(project_id='tecs',
                                     user_id=common.fake_action_0['user_id'],
                                     offset=6, limit=15,
                                     search=my_search)
        self.assertEqual([common.fake_action_0, common.fake_action_0], res)
        self.eng.action_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_action_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    def test_get_action_with_userid_and_search_return_empty_list(self):
        self.eng.action_manager.search.return_value = []
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_action(project_id='tecs',
                                     user_id=common.fake_action_0['user_id'],
                                     offset=6, limit=15,
                                     search=my_search)
        self.assertEqual([], res)
        self.eng.action_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_action_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    @patch('freezer_api.common.elasticv2_utils.ActionDoc')
    def test_add_action_ok(self, mock_actiondoc):
        mock_actiondoc.create.return_value = common.get_fake_action_0()
        self.eng.action_manager.insert.return_value = (True, 1)
        res = self.eng.add_action(project_id='tecs',
                                  user_id=common.fake_action_0['user_id'],
                                  doc=common.get_fake_action_0())
        self.assertEqual(common.fake_action_0['action_id'], res)
        self.eng.action_manager.insert.assert_called_with(
            common.fake_action_0, common.fake_action_0['action_id']
        )

    def test_add_action_raises_StorageEngineError_when_manager_insert_raises(
            self):
        self.eng.action_manager.get.return_value = None
        self.eng.action_manager.insert.side_effect = (
            exceptions.StorageEngineError('regular test failure')
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.add_action,
                          project_id='tecs',
                          user_id=common.fake_action_0['user_id'],
                          doc=common.get_fake_action_0())

    def test_delete_action_ok(self):
        self.eng.action_manager.delete.return_value = common.fake_action_0[
            'action_id']
        res = self.eng.delete_action(
            project_id='tecs',
            user_id=common.fake_action_0['action_id'],
            action_id=common.fake_action_0['action_id'])
        self.assertEqual(common.fake_action_0['action_id'], res)

    def test_delete_client_raises_StorageEngineError_when_es_delete_raises(
            self):
        self.eng.action_manager.delete.side_effect = (
            exceptions.StorageEngineError()
        )
        self.assertRaises(exceptions.StorageEngineError,
                          self.eng.delete_action,
                          project_id='tecs',
                          user_id=common.fake_action_0['action_id'],
                          action_id=common.fake_action_0['action_id'])

    def test_update_action_raises_DocumentNotFound_when_doc_not_exists(self):
        self.eng.action_manager.get.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        patch = {'action_id': 'black_milk'}
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_action,
                          project_id='tecs',
                          user_id=common.fake_action_0['action_id'],
                          action_id=common.fake_action_0['action_id'],
                          patch_doc=patch)

    def test_update_action_raises_DocumentNotFound_when_update_raises(
            self):
        self.eng.action_manager.get.return_value = common.get_fake_action_0()
        patch = {'action_id': 'black_milk'}
        self.eng.action_manager.update.side_effect = (
            exceptions.DocumentNotFound('regular test failure')
        )
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_action,
                          project_id='tecs',
                          user_id=common.fake_action_0['action_id'],
                          action_id=common.fake_action_0['action_id'],
                          patch_doc=patch)

    def test_update_action_returns_new_doc_version(self):
        self.eng.action_manager.get.return_value = common.get_fake_action_0()
        patch = {'action_id': 'group_four'}
        self.eng.action_manager.update.return_value = 11
        res = self.eng.update_action(
            project_id='tecs',
            user_id=common.fake_action_0['action_id'],
            action_id=common.fake_action_0['action_id'],
            patch_doc=patch)
        self.assertEqual(11, res)

    def test_replace_action_raises_AccessForbidden_when_action_manager_raises(
            self):
        self.eng.action_manager.get.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        self.eng.action_manager.insert.return_value = (True, 3)
        self.assertRaises(exceptions.AccessForbidden, self.eng.replace_action,
                          project_id='tecs',
                          user_id=common.fake_action_0['action_id'],
                          action_id=common.fake_action_0['action_id'],
                          doc=common.get_fake_action_0())

    # def test_replace_action_returns_ok_when_doc_is_new(self):
    #     self.eng.action_manager.get.side_effect = (
    #         exceptions.DocumentNotFound('regular test failure')
    #     )
    #     self.eng.action_manager.insert.return_value = (True, 1)
    #     res = self.eng.replace_action(
    #         user_id=common.fake_action_0['action_id'],
    #         action_id=common.fake_action_0['action_id'],
    #         doc=common.get_fake_action_0())
    #     self.assertEqual(res, 1)
    #
    # def test_replace_action_returns_version_1_when_doc_is_overwritten(
    #         self):
    #     self.eng.action_manager.get.return_value = common.get_fake_action_0()
    #     self.eng.action_manager.insert.return_value = (False, 3)
    #     res = self.eng.replace_action(
    #         user_id=common.fake_action_0['action_id'],
    #         action_id=common.fake_action_0['action_id'],
    #         doc=common.get_fake_action_0())
    #     self.assertEqual(res, 3)


class TestElasticSearchEngine_session(unittest.TestCase, ElasticSearchDB):
    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_elasticsearch, mock_logging):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)
        self.eng.session_manager = mock.Mock()

    def test_get_session_userid_and_session_id_return_doc(self):
        self.eng.session_manager.get.return_value = common.get_fake_session_0()
        res = self.eng.get_session(project_id='tecs',
                                   user_id=common.fake_session_0['user_id'],
                                   session_id=common.fake_session_0[
                                       'session_id'])
        self.assertEqual(common.fake_session_0, res)
        self.eng.session_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            doc_id=common.fake_session_0['session_id'])

    def test_get_session_userid_and_session_id_return_none(self):
        self.eng.session_manager.get.return_value = None
        res = self.eng.get_session(project_id='tecs',
                                   user_id=common.fake_session_0['user_id'],
                                   session_id=common.fake_session_0[
                                       'session_id'])
        self.assertIsNone(res)
        self.eng.session_manager.get.assert_called_with(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            doc_id=common.fake_session_0['session_id'])

    def test_get_session_with_userid_and_search_return_list(self):
        self.eng.session_manager.search.return_value = \
            [common.fake_session_0, common.fake_session_0]
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_session(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            offset=6, limit=15,
            search=my_search)
        self.assertEqual([common.fake_session_0, common.fake_session_0], res)
        self.eng.session_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    def test_get_session_with_userid_and_search_return_empty_list(self):
        self.eng.session_manager.search.return_value = []
        my_search = {'match': [{'some_field': 'some text'},
                               {'description': 'some other text'}]}
        res = self.eng.search_session(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            offset=6, limit=15,
            search=my_search)
        self.assertEqual([], res)
        self.eng.session_manager.search.assert_called_with(
            project_id='tecs',
            user_id=common.fake_session_0['user_id'],
            search=my_search,
            limit=15, offset=6)

    @patch('freezer_api.common.elasticv2_utils.SessionDoc')
    def test_add_session_ok(self, mock_sessiondoc):
        mock_sessiondoc.create.return_value = common.get_fake_session_0()
        self.eng.session_manager.insert.return_value = (True, 1)
        res = self.eng.add_session(project_id='tecs',
                                   user_id=common.fake_session_0['user_id'],
                                   doc=common.get_fake_session_0())
        self.assertEqual(common.fake_session_0['session_id'], res)
        self.eng.session_manager.insert.assert_called_with(
            common.fake_session_0,
            common.fake_session_0['session_id'])

    def test_add_session_raises_StorageEngineError_when_manager_insert_raises(
            self):
        self.eng.session_manager.get.return_value = None
        self.eng.session_manager.insert.side_effect = (
            exceptions.StorageEngineError('regular test failure')
        )
        self.assertRaises(exceptions.StorageEngineError, self.eng.add_session,
                          project_id='tecs',
                          user_id=common.fake_session_0['user_id'],
                          doc=common.get_fake_session_0())

    def test_delete_session_ok(self):
        self.eng.session_manager.delete.return_value = common.fake_session_0[
            'session_id']
        res = self.eng.delete_session(
            project_id='tecs',
            user_id=common.fake_session_0['session_id'],
            session_id=common.fake_session_0['session_id'])
        self.assertEqual(common.fake_session_0['session_id'], res)

    def test_delete_client_raises_StorageEngineError_when_es_delete_raises(
            self):
        self.eng.session_manager.delete.side_effect = (
            exceptions.StorageEngineError()
        )
        self.assertRaises(exceptions.StorageEngineError,
                          self.eng.delete_session,
                          project_id='tecs',
                          user_id=common.fake_session_0['session_id'],
                          session_id=common.fake_session_0['session_id'])

    def test_update_session_raises_DocumentNotFound_when_doc_not_exists(self):
        self.eng.session_manager.get.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        patch = {'session_id': 'black_milk'}
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_session,
                          project_id='tecs',
                          user_id=common.fake_session_0['session_id'],
                          session_id=common.fake_session_0['session_id'],
                          patch_doc=patch)

    def test_update_session_raises_DocumentNotFound_when_update_raises(
            self):
        self.eng.session_manager.get.return_value = common.get_fake_session_0()
        patch = {'session_id': 'black_milk'}
        self.eng.session_manager.update.side_effect = (
            exceptions.DocumentNotFound('regular test failure')
        )
        self.assertRaises(exceptions.DocumentNotFound, self.eng.update_session,
                          project_id='tecs',
                          user_id=common.fake_session_0['session_id'],
                          session_id=common.fake_session_0['session_id'],
                          patch_doc=patch)

    def test_update_session_returns_new_doc_version(self):
        self.eng.session_manager.get.return_value = common.get_fake_session_0()
        patch = {'session_id': 'group_four'}
        self.eng.session_manager.update.return_value = 11
        res = self.eng.update_session(
            project_id='tecs',
            user_id=common.fake_session_0['session_id'],
            session_id=common.fake_session_0['session_id'],
            patch_doc=patch)
        self.assertEqual(11, res)

    def test_replace_session_raises_AccessForbidden_when_session_manager_raise(
            self):
        self.eng.session_manager.get.side_effect = exceptions.AccessForbidden(
            'regular test failure')
        self.eng.session_manager.insert.return_value = (True, 3)
        self.assertRaises(exceptions.AccessForbidden, self.eng.replace_session,
                          project_id='tecs',
                          user_id=common.fake_session_0['session_id'],
                          session_id=common.fake_session_0['session_id'],
                          doc=common.get_fake_session_0())

    def test_replace_session_returns_ok_when_doc_is_new(self):
        self.eng.session_manager.get.side_effect = exceptions.DocumentNotFound(
            'regular test failure')
        self.eng.session_manager.insert.return_value = (True, 1)
        res = self.eng.replace_session(
            project_id='tecs',
            user_id=common.fake_session_0['session_id'],
            session_id=common.fake_session_0['session_id'],
            doc=common.get_fake_session_0())
        self.assertEqual(1, res)

    def test_replace_session_returns_version_1_when_doc_is_overwritten(self):
        self.eng.session_manager.get.return_value = common.get_fake_session_0()
        self.eng.session_manager.insert.return_value = (False, 3)
        res = self.eng.replace_session(
            project_id='tecs',
            user_id=common.fake_session_0['session_id'],
            session_id=common.fake_session_0['session_id'],
            doc=common.get_fake_session_0())
        self.assertEqual(3, res)


class TestElasticSearchEngineV2(unittest.TestCase, ElasticSearchDB):

    @patch('freezer_api.storage.elasticv2.logging')
    @patch('freezer_api.storage.elasticv2.elasticsearch')
    def setUp(self, mock_logging, mock_elasticsearch):
        backend = 'elasticsearch'
        grp = cfg.OptGroup(backend)
        CONF.register_group(grp)
        CONF.register_opts(self._ES_OPTS, group=backend)
        mock_elasticsearch.Elasticsearch.return_value = mock.Mock()
        kwargs = {'hosts': 'http://elasticservaddr:1997'}
        self.eng = elastic.ElasticSearchEngineV2(backend=backend)
        self.eng.init(index='freezer', **kwargs)

    def test_raise_validate_opts_when_ca_certs_file_not_exist(self):
        self.eng.conf.update({'ca_certs': 'invalid_ca_certs_file'})
        self.assertRaises(Exception,
                          self.eng._validate_opts)



"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from freezer_api.api.v1 import actions
from freezer_api.api.v1 import backups
from freezer_api.api.v1 import clients
from freezer_api.api.v1 import homedoc
from freezer_api.api.v1 import jobs
from freezer_api.api.v1 import sessions


VERSION = {
    'id': 'v1',
    'status': 'DEPRECATED',
    'updated': '2018-02-21T13:30:00',
    'links': [
        {
            'href': '{0}v1/',
            'rel': 'self'
        }
    ]
}


def public_endpoints(storage_driver):
    return [
        ('/',
         homedoc.Resource()),

        ('/backups',
         backups.BackupsCollectionResource(storage_driver)),

        ('/backups/{backup_id}',
         backups.BackupsResource(storage_driver)),

        ('/clients',
         clients.ClientsCollectionResource(storage_driver)),

        ('/clients/{client_id}',
         clients.ClientsResource(storage_driver)),

        ('/jobs',
         jobs.JobsCollectionResource(storage_driver)),

        ('/jobs/{job_id}',
         jobs.JobsResource(storage_driver)),

        ('/jobs/{job_id}/event',
         jobs.JobsEvent(storage_driver)),

        ('/actions',
         actions.ActionsCollectionResource(storage_driver)),

        ('/actions/{action_id}',
         actions.ActionsResource(storage_driver)),

        ('/sessions',
         sessions.SessionsCollectionResource(storage_driver)),

        ('/sessions/{session_id}',
         sessions.SessionsResource(storage_driver)),

        ('/sessions/{session_id}/action',
         sessions.SessionsAction(storage_driver)),

        ('/sessions/{session_id}/jobs/{job_id}',
         sessions.SessionsJob(storage_driver)),

    ]



"""
(c) Copyright 2015-2016 Hewlett-Packard Enterprise Company L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import functools

from oslo_config import cfg
from oslo_policy import opts
from oslo_policy import policy

from freezer_api.common import exceptions
from freezer_api.common import policies

# TODO(gmann): Remove setting the default value of config policy_file
# once oslo_policy change the default value to 'policy.yaml'.
# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
DEFAULT_POLICY_FILE = 'policy.yaml'
opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)

ENFORCER = None


def setup_policy(conf):
    global ENFORCER

    if not ENFORCER:
        ENFORCER = policy.Enforcer(conf)
        ENFORCER.register_defaults(policies.list_rules())
        ENFORCER.load_rules()


def enforce(rule):

    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            ctx = args[1].env['freezer.context']
            ENFORCER.enforce(rule, {}, ctx.to_dict(), do_raise=True,
                             exc=exceptions.AccessForbidden)
            return func(*args, **kwargs)
        return handler

    return decorator



# (c) Copyright 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from freezer_api import context


class TestFreezerContext(unittest.TestCase):

    def setUp(self):
        self.ctxt = context.FreezerContext('token01', 'user01', 'tenant01',
                                           is_admin=False,
                                           roles=['_member', 'backup_admin'])

    def test_not_admin(self):
        ctxt = context.get_current()
        self.assertEqual(False, ctxt.is_admin)

    def test_privileges_elevated(self):
        self.ctxt = self.ctxt.elevated('yes')
        self.assertTrue(self.ctxt.is_admin)
        self.assertEqual('yes', self.ctxt.show_deleted)

    def test_admin_context(self):
        admin_ctxt = context.get_admin_context(show_deleted='yes')
        self.assertTrue(admin_ctxt.is_admin)
        self.assertEqual('yes', admin_ctxt.show_deleted)

    def test_from_dict(self):
        ctxt_dict = {'user': 'user02',
                     'tenant': 'tenant03',
                     'roles': ['_member', 'member']}
        ctxt = context.FreezerContext.from_dict(ctxt_dict)
        self.assertFalse(ctxt.is_admin)
        # self.assertEqual('user02', ctxt.user)
        # self.assertEqual(ctxt_dict.get('tenant'), ctxt.tenant)



"""Freezer swift.py related tests

(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import falcon
from unittest import mock

from freezer_api.api.v1 import backups
from freezer_api.common import exceptions
from freezer_api.tests.unit import common


class TestBackupsCollectionResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestBackupsCollectionResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = {
            'X-User-ID': common.fake_data_0_user_id}
        self.mock_req.status = falcon.HTTP_200
        self.resource = backups.BackupsCollectionResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_empty_list(self):
        self.mock_db.search_backup.return_value = []
        expected_result = {'backups': []}
        self.resource.on_get(self.mock_req, self.mock_req)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_get_return_correct_list(self):
        self.mock_db.search_backup.return_value = [
            common.fake_data_0_backup_metadata]
        expected_result = {'backups': [common.fake_data_0_backup_metadata]}
        self.resource.on_get(self.mock_req, self.mock_req)
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_post_raises_when_missing_body(self):
        self.mock_db.add_backup.return_value = [
            common.fake_data_0_wrapped_backup_metadata['backup_id']]
        self.assertRaises(exceptions.BadDataFormat, self.resource.on_post,
                          self.mock_req, self.mock_req)

    def test_on_post_inserts_correct_data(self):
        self.mock_json_body.return_value = common.fake_data_0_backup_metadata
        self.mock_db.add_backup.return_value = (
            common.fake_data_0_wrapped_backup_metadata['backup_id']
        )
        self.resource.on_post(self.mock_req, self.mock_req)
        expected_result = {
            'backup_id': common.fake_data_0_wrapped_backup_metadata[
                'backup_id']}
        self.assertEqual(expected_result, self.mock_req.body)
        self.assertEqual(falcon.HTTP_201, self.mock_req.status)


class TestBackupsResource(common.FreezerBaseTestCase):
    def setUp(self):
        super(TestBackupsResource, self).setUp()
        self.mock_db = mock.Mock()
        self.mock_req = mock.MagicMock()
        self.mock_req.env.__getitem__.side_effect = common.get_req_items
        self.mock_req.get_header.return_value = {
            'X-User-ID': common.fake_data_0_user_id}
        self.mock_req.status = falcon.HTTP_200
        self.resource = backups.BackupsResource(self.mock_db)
        self.mock_json_body = mock.Mock()
        self.mock_json_body.return_value = {}
        self.resource.json_body = self.mock_json_body

    def test_on_get_return_no_result_and_404_when_not_found(self):
        self.mock_db.get_backup.return_value = []
        self.mock_req.body = None
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_data_0_wrapped_backup_metadata[
                                 'backup_id'])
        self.assertIsNone(self.mock_req.body)
        self.assertEqual(falcon.HTTP_404, self.mock_req.status)

    def test_on_get_return_correct_data(self):
        self.mock_db.get_backup.return_value = [
            common.fake_data_0_wrapped_backup_metadata]
        expected_result = [common.fake_data_0_wrapped_backup_metadata]
        self.resource.on_get(self.mock_req, self.mock_req,
                             common.fake_data_0_wrapped_backup_metadata[
                                 'backup_id'])
        result = self.mock_req.body
        self.assertEqual(expected_result, result)
        self.assertEqual(falcon.HTTP_200, self.mock_req.status)

    def test_on_delete_removes_proper_data(self):
        self.resource.on_delete(self.mock_req, self.mock_req,
                                common.fake_data_0_backup_id)
        result = self.mock_req.body
        expected_result = {'backup_id': common.fake_data_0_backup_id}
        self.assertEqual(falcon.HTTP_204, self.mock_req.status)
        self.assertEqual(expected_result, result)



# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2014 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_db.sqlalchemy import models
from oslo_utils import timeutils
from sqlalchemy import Column, Integer, String, Text, TIMESTAMP
from sqlalchemy import BLOB
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import ForeignKey, DateTime, Boolean
from sqlalchemy.orm import relationship


BASE = declarative_base()


class FreezerBase(models.TimestampMixin,
                  models.ModelBase):
    """Base class for Freezer Models."""

    __table_args__ = {'mysql_engine': 'InnoDB'}

    deleted_at = Column(DateTime)
    deleted = Column(Boolean, default=False)
    backup_metadata = None

    @staticmethod
    def delete_values():
        return {'deleted': True,
                'deleted_at': timeutils.utcnow()}

    def delete(self, session):
        """Delete this object."""
        updated_values = self.delete_values()
        self.update(updated_values)
        self.save(session=session)
        return updated_values


class Client(BASE, FreezerBase):
    """Represents a scheduler of the freezer action."""

    __tablename__ = 'clients'
    id = Column(String(255), primary_key=True)
    project_id = Column(String(36))
    user_id = Column(String(64), nullable=False)
    client_id = Column(String(255), nullable=False)
    hostname = Column(String(128))
    description = Column(String(255))
    uuid = Column(String(36), nullable=False)


class Action(BASE, FreezerBase):
    """Represents freezer action."""
    # The field backup_metadata is json, including :
    # hostname ,snapshot ,storage ,dry_run , lvm_auto_snap, lvm_dirmount,
    # lvm_snapname max_level , max_priority, max_segment_size , mode,
    # mysql_conf, path_to_backup,  remove_older_than restore_abs_path
    # restore_from_host, ssh_host , ssh_key , ssh_username, ssh_port , proxy
    # no_incremental, overwrite , nova_inst_id , engine_name
    # restore_from_date , command , incremental

    __tablename__ = 'actions'
    id = Column(String(36), primary_key=True)
    action = Column(String(255), nullable=False)
    project_id = Column(String(36))
    user_id = Column(String(64), nullable=False)
    actionmode = Column(String(255))
    path_to_backup = Column(String(255))
    backup_name = Column(String(255))
    container = Column(String(255))
    timeout = Column(Integer)
    priority = Column(Integer)
    max_retries_interval = Column(Integer, default=6)
    max_retries = Column(Integer, default=5)
    mandatory = Column(Boolean, default=False)
    log_file = Column(String(255))
    backup_metadata = Column(Text)


class Session(BASE, FreezerBase):
    """Represents freezer session."""

    __tablename__ = 'sessions'
    id = Column(String(36), primary_key=True)
    session_tag = Column(Integer, default=0)
    description = Column(String(255))
    hold_off = Column(Integer, default=30)
    schedule = Column(Text)
    job = Column(Text)
    project_id = Column(String(36))
    user_id = Column(String(36), nullable=False)
    time_start = Column(Integer, default=-1)
    time_end = Column(Integer, default=-1)
    time_started = Column(Integer, default=-1)
    time_ended = Column(Integer, default=-1)
    status = Column(String(255))
    result = Column(String(255))


class Job(BASE, FreezerBase):
    """Represents freezer job."""

    __tablename__ = 'jobs'
    id = Column(String(36), primary_key=True)
    project_id = Column(String(36))
    user_id = Column(String(36), nullable=False)
    schedule = Column(Text)
    client_id = Column(String(255), ForeignKey('clients.id'), nullable=False)
    session_id = Column(String(36), ForeignKey('sessions.id'), nullable=False)
    session_tag = Column(Integer, default=0)
    description = Column(String(255))
    job_actions = Column(Text)
    client = relationship(Client, backref='jobs',
                          foreign_keys=client_id,
                          primaryjoin='and_('
                          'Job.client_id == Client.id,'
                          'Job.deleted == False)')
    session = relationship(Session, backref='jobs',
                           foreign_keys=session_id,
                           primaryjoin='and_('
                           'Job.session_id == Session.id,'
                           'Job.deleted == False)')


class ActionReport(BASE, FreezerBase):
    __tablename__ = 'action_reports'
    id = Column(String(36), primary_key=True)
    project_id = Column(String(36))
    user_id = Column(String(64), nullable=False)
    result = Column(String(255))
    time_elapsed = Column(String(255))
    report_date = Column(TIMESTAMP)
    log = Column(BLOB)


class Backup(BASE, FreezerBase):
    """Represents freezer Backup."""
    # The field backup_metadata is json, including :
    # nova_inst_id, engine_name, storage, remove_older_than, restore_from_date,
    # command, incremental, restore_abs_path, etc

    __tablename__ = 'backups'
    id = Column(String(36), primary_key=True)
    job_id = Column(String(36))
    project_id = Column(String(36))
    user_id = Column(String(64), nullable=False)
    user_name = Column(String(64), nullable=False)
    backup_metadata = Column(Text)


def register_models(engine):
    _models = (Client, Action, Job, Session,
               ActionReport, Backup)
    for _model in _models:
        _model.metadata.create_all(engine)


def unregister_models(engine):
    _models = (Client, Action, Job, Session,
               ActionReport, Backup)
    for _model in _models:
        _model.metadata.drop_all(engine)


def get_tables(engine):
    from sqlalchemy import MetaData
    _meta = MetaData()
    _meta.reflect(engine)
    return _meta.tables.keys()



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import elasticsearch
from oslo_config import cfg
from oslo_log import log

from freezer_api.common import db_mappings


CONF = cfg.CONF
LOG = log.getLogger(__name__)

DEFAULT_INDEX = 'freezer'
DEFAULT_REPLICAS = 0


class ElasticSearchManager(object):
    """
    Managing ElasticSearch mappings operations
    Sync: create mappings
    Update: Update mappings
    remove: deletes the mappings
    show: print out all the mappings
    """

    def __init__(self, **options):
        self.mappings = db_mappings.get_mappings().copy()
        self.conf = options.copy()
        self.index = self.conf['index']

        self.elk = elasticsearch.Elasticsearch(**options)
        # check if the cluster is up or not !
        if not self.elk.ping():
            raise Exception('ElasticSearch cluster is not available. '
                            'Cannot ping it')
        # clear the index cache
        try:
            self.elk.indices.clear_cache(index=self.conf['index'])
        except Exception as e:
            LOG.warning(e)

    def _check_index_exists(self, index):
        LOG.info('check if index: {0} exists or not'.format(index))
        try:
            return self.elk.indices.exists(index=index)
        except elasticsearch.TransportError:
            raise

    def _check_mapping_exists(self, mappings):
        LOG.info('check if mappings: {0} exists or not'.format(mappings))
        return self.elk.indices.exists_type(index=self.index,
                                            doc_type=mappings)

    def get_required_mappings(self):
        """
        This function checks if the user chooses a certain mappings or not.
        If the user has chosen a certain mappings it will return these mappings
        only If not it will return all mappings to be updated
        :return:
        """
        # check if the user asked to update only one mapping ( -m is provided )
        mappings = {}
        if self.conf['select_mapping']:
            if self.conf['select_mapping'] not in self.mappings.keys():
                raise Exception(
                    'Selected mappings {0} does not exists. Please, choose '
                    'one of {1}'.format(self.conf['select_mapping'],
                                        self.mappings.keys()
                                        )
                )
            mappings[self.conf['select_mapping']] = \
                self.mappings.get(self.conf['select_mapping'])
        else:
            mappings = self.mappings
        return mappings

    def db_sync(self):
        """
        Create or update elasticsearch db mappings
        steps:
        1) check if mappings exists
        2) remove mapping if erase is passed
        3) update mappings if - y is passed
        4) if update failed ask for permission to remove old mappings
        5) try to update again
        6) if update succeeded exit :)
        :return:
        """
        # check if erase provided remove mappings first
        if self.conf.get('erase'):
            self.remove_mappings()

        # check if index does not exists create it
        if not self._check_index_exists(self.index):
            self._create_index()

        _mappings = self.get_required_mappings()
        # create/update one by one
        for doc_type, body in _mappings.items():
            check = self.create_one_mapping(doc_type, body)
            if check:
                print("Creating or Updating {0} is {1}".format(
                    doc_type, check.get('acknowledged')))
            else:
                print("Couldn't update {0}. Request returned {1}".format(
                    doc_type, check.get('acknowledged')))

    def _create_index(self):
        """
        Create the index that will allow us to put the mappings under it
        :return: {u'acknowledged': True} if success or None if index exists
        """
        if not self._check_index_exists(index=self.index):
            body = {
                'number_of_replicas':
                    self.conf['number_of_replicas'] or DEFAULT_REPLICAS
            }
            return self.elk.indices.create(index=self.index, body=body)

    def delete_index(self):
        return self.elk.indices.delete(index=self.index)

    def create_one_mapping(self, doc_type, body):
        """
        Create one document type and update its mappings
        :param doc_type: the document type to be created jobs, clients, backups
        :param body: the structure of the document
        :return: dict
        """
        # check if doc_type exists or not
        if self._check_mapping_exists(doc_type):
            do_update = self.prompt(
                '[[[ {0} ]]] already exists in index => {1}'
                ' <= Do you want to update it ? (y/n) '.format(doc_type,
                                                               self.index)
            )
            if do_update:
                # Call elasticsearch library and put the mappings
                return self.elk.indices.put_mapping(doc_type=doc_type,
                                                    body=body,
                                                    index=self.index
                                                    )
            else:
                return {'acknowledged': False}
        return self.elk.indices.put_mapping(doc_type=doc_type, body=body,
                                            index=self.index)

    def remove_one_mapping(self, doc_type):
        """
        Removes one mapping at a time
        :param doc_type: document type to be removed
        :return: dict
        """
        LOG.info('Removing mapping {0} from index {1}'.format(doc_type,
                                                              self.index))
        try:
            return self.elk.indices.delete_mapping(self.index,
                                                   doc_type=doc_type)
        except Exception:
            raise

    def remove_mappings(self):
        """
        Remove mappings from elasticsearch
        :return: dict
        """
        # check if index doesn't exist return
        if not self._check_index_exists(index=self.index):
            print("Index {0} doesn't exists.".format(self.index))
            return
        # remove mappings
        self.delete_index()

    def update_mappings(self):
        """
        Update mappings
        :return: dict
        """
        self.conf['yes'] = True
        return self.db_sync()

    def show_mappings(self):
        """
        Print existing mappings in an index
        :return: dict
        """
        # check if index doesn't exist return
        if not self._check_index_exists(index=self.index):
            LOG.debug("Index {0} doesn't exists.".format(self.index))
            return
        return self.elk.indices.get_mapping(index=self.index)

    def update_settings(self):
        """
        Update number of replicas
        :return: dict
        """
        body = {
            'number_of_replicas':
                self.conf['number_of_replicas'] or DEFAULT_REPLICAS
        }
        return self.elk.indices.put_settings(body=body, index=self.index)

    def prompt(self, message):
        """
        Helper function that is being used to ask the user for confirmation
        :param message: Message to be printed (To ask the user to confirm ...)
        :return: True or False
        """
        if self.conf['yes']:
            return self.conf['yes']
        while True:
            ans = input(message)
            if ans.lower() == 'y':
                return True
            elif ans.lower() == 'n':
                return False



"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
(C) Copyright 2016-2018 Hewlett Packard Enterprise Development Company LP
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import sys

from oslo_log import log
from oslo_utils import importutils
from stevedore import driver

LOG = log.getLogger(__name__)


_DB_DRIVER_NAMESPACE = "freezer.db.backends"


def _load_class_by_alias_or_classname(namespace, name):
    """Load class using stevedore alias or the class name
    :param namespace: namespace where the alias is defined
    :param name: alias or class name of the class to be loaded
    :returns: class if calls can be loaded
    :raises ImportError if class cannot be loaded
    """

    if not name:
        LOG.error("Alias or class name is not set")
        raise ImportError("Class not found.")
    try:
        # Try to resolve class by alias
        mgr = driver.DriverManager(
            namespace, name, warn_on_missing_entrypoint=False)
        class_to_load = mgr.driver
    except RuntimeError:
        e1_info = sys.exc_info()
        # Fallback to class name
        try:
            class_to_load = importutils.import_class(name)
        except (ImportError, ValueError):
            LOG.error("Error loading class by alias",
                      exc_info=e1_info)
            LOG.error("Error loading class by class name",
                      exc_info=True)
            raise ImportError("Class not found.")
    return class_to_load


def get_db_driver(name, backend):
    """
    Loads database driver
    :param name: name of the database driver.
    :return: Instance of the driver class
    """
    driver_class = _load_class_by_alias_or_classname(_DB_DRIVER_NAMESPACE,
                                                     name)
    return driver_class(backend=backend)



"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import falcon
from oslo_log import log
import webob.dec
import webob.exc

from oslo_serialization import jsonutils as json

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api import context

LOG = log.getLogger(__name__)


class Middleware(object):
    """
    WSGI wrapper for all freezer middlewares. Use this will allow to manage
    middlewares through paste
    """

    def __init__(self, app):
        self.app = app

    def process_request(self, req):
        """
        implement this function in your middleware to change the request
        if the function return None the request will be handled in the next
        level functions
        """
        return None

    def process_response(self, response):
        """
        Implement this to modify your response
        """
        return response

    @classmethod
    def factory(cls, global_conf, **local_conf):
        def filter(app):
            return cls(app)

        return filter

    @webob.dec.wsgify
    def __call__(self, req, params=None):
        response = self.process_request(req)
        if response:
            return response
        response = req.get_response(self.app)
        response.req = req
        try:
            return self.process_response(response)
        except webob.exc.HTTPException as e:
            LOG.error(e)
            return e


class HookableMiddlewareMixin(object):
    """Provides methods to extract before and after hooks from WSGI Middleware

    Prior to falcon 0.2.0b1, it's necessary to provide falcon with middleware
    as "hook" functions that are either invoked before (to process requests)
    or after (to process responses) the API endpoint code runs.

    This mixin allows the process_request and process_response methods from a
    typical WSGI middleware object to be extracted for use as these hooks, with
    the appropriate method signatures.
    """

    def as_before_hook(self):
        """Extract process_request method as "before" hook

        :return: before hook function
        """

        # Need to wrap this up in a closure because the parameter counts
        # differ
        def before_hook(req, resp, params=None):
            return self.process_request(req, resp)

        try:
            return before_hook
        except AttributeError as ex:
            # No such method, we presume.
            message_template = ("Failed to get before hook from middleware "
                                "{0} - {1}")
            message = message_template.format(self.__name__, ex.message)
            LOG.error(message)
            LOG.exception(ex)
            raise freezer_api_exc.FreezerAPIException(message)

    def as_after_hook(self):
        """Extract process_response method as "after" hook

        :return: after hook function
        """

        # Need to wrap this up in a closure because the parameter counts
        # differ
        def after_hook(req, resp, resource=None):
            return self.process_response(req, resp, resource)

        try:
            return after_hook
        except AttributeError as ex:
            # No such method, we presume.
            message_template = ("Failed to get after hook from middleware "
                                "{0} - {1}")
            message = message_template.format(self.__name__, ex.message)
            LOG.error(message)
            LOG.exception(ex)
            raise freezer_api_exc.FreezerAPIException(message)


class RequireJSON(HookableMiddlewareMixin, object):
    def process_request(self, req, resp):
        if not req.client_accepts_json:
            raise falcon.HTTPNotAcceptable(
                'Freezer-api only supports responses encoded as JSON.',
                href='http://docs.examples.com/api/json')


class JSONTranslator(HookableMiddlewareMixin, object):
    def process_response(self, req, resp, resource, req_succeeded=True):
        if not hasattr(resp, 'body'):
            return
        if isinstance(resp.data, dict):
            resp.data = json.dumps(resp.data)

        if isinstance(resp.body, dict):
            resp.body = json.dumps(resp.body)


class BaseContextMiddleware(Middleware):
    def process_response(self, response):
        try:
            request_id = response.req.context.request_id
        except AttributeError:
            LOG.warning('Unable to retrieve request id from context')
        else:
            # For python 3 compatibility need to use bytes type
            prefix = b'req-' if isinstance(request_id, bytes) else 'req-'

            if not request_id.startswith(prefix):
                request_id = prefix + request_id

            response.headers['x-openstack-request-id'] = request_id
        return response


class ContextMiddleware(BaseContextMiddleware):
    """
    Simple WSGI app to support HAProxy polling.
    If the requested url matches the configured path it replies
    with a 200 otherwise passes the request to the inner app
    """

    def get_context(self, req):
        token = req.headers.get('X-Auth-Token')
        userid = req.headers.get('X-User-Id')
        domainid = req.headers.get('X-Domain-Id')
        tenantid = req.headers.get('X-Tenant-Id')
        user_domain_id = req.headers.get('X-User-Domain-Id')
        projectid = req.headers.get('X-Project-Id')
        project_domain_id = req.headers.get('X-Project-Domain-Id')
        request_id = req.headers.get('X-Openstack-Request-ID')
        roles_header = req.headers.get('X-Roles', '')
        roles = [r.strip().lower() for r in roles_header.split(',')]
        is_admin = 'admin' in roles

        return context.FreezerContext(auth_token=token,
                                      user=userid,
                                      tenant=tenantid or projectid,
                                      domain=domainid,
                                      user_domain=user_domain_id,
                                      project_domain=project_domain_id,
                                      is_admin=is_admin,
                                      request_id=request_id,
                                      resource_uuid=None,
                                      roles=roles
                                      )

    def process_request(self, req):
        if req.headers.get('X-Identity-Status') == "Confirmed":
            # falcon is overwriting the context variable in the request
            # so we need to set the context in env variable
            req.context = self.get_context(req)
            req.environ['freezer.context'] = req.context
        else:
            raise webob.exc.HTTPUnauthorized()



"""
Copyright 2016 Hewlett-Packard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from unittest.mock import patch

import falcon

from freezer_api import service
from freezer_api.tests.unit import common


class TestService(common.FreezerBaseTestCase):
    def setUp(self):
        self.falcon_versions_hooks = ['0.1.6', '0.1.7', '0.1.8', '0.1.9',
                                      '0.1.10']
        self.falcon_versions_middleware = ['0.2.0', '0.3.0', '1.0.0']
        super(TestService, self).setUp()

    @patch('freezer_api.cmd.api.v1')
    @patch('freezer_api.cmd.api.manager')
    @patch('freezer_api.cmd.api.falcon')
    def test_on_new_falcon_builds_v1(self, mock_falcon, mock_driver, mock_v1):
        """Test that falcon versions that should use new middleware syntax do so

        :param mock_falcon: The falcon import freezer-api will try to
        start up
        :param mock_driver: Database driver
        :param mock_v1: List of endpoints for v1 for the freezer API.
        """
        mock_driver.get_db.return_value = None
        mock_v1.endpoints = []

        # Iterate through all of the versions of falcon that should be using
        # the old before=,after= invocation and ensure that freezer-api isn't
        # trying to invoke it in the old style.
        for version_string in self.falcon_versions_middleware:
            version_attribute = '__version__' if hasattr(
                falcon, '__version__') else 'version'
            with patch('falcon.' + version_attribute, version_string):
                # Attempt to invoke a mocked version of falcon to see what args
                # it was called with
                service.freezer_appv1_factory(None)

                # Check kwargs to see if the correct arguments are being passed
                _, kwargs = mock_falcon.API.call_args

                named_args = kwargs.keys()

                self.assertNotIn('before', named_args)
                self.assertNotIn('after', named_args)
                self.assertIn('middleware', named_args)

    @patch('freezer_api.cmd.api.v2')
    @patch('freezer_api.cmd.api.manager')
    @patch('freezer_api.cmd.api.falcon')
    def test_on_old_falcon_builds_v2(self, mock_falcon, mock_driver, mock_v2):
        """Test that falcon versions that should use old middleware syntax do so

        :param mock_falcon: The falcon import freezer-api will try to
        start up
        :param mock_driver: Database driver
        :param mock_v1: List of endpoints for v1 for the freezer API.
        """
        mock_driver.get_db.return_value = None
        mock_v2.endpoints = []

        # Iterate through all of the versions of falcon that should be using
        # the old before=,after= invocation and ensure that freezer-api isn't
        # trying to invoke it in the old style.
        for version_string in self.falcon_versions_hooks:
            version_attribute = '__version__' if hasattr(
                falcon, '__version__') else 'version'
            with patch('falcon.' + version_attribute, version_string):
                # Attempt to invoke a mocked version of falcon to see what args
                # it was called with
                service.freezer_appv2_factory(None)

                # Check kwargs to see if the correct arguments are being passed
                _, named_args = mock_falcon.API.call_args

                self.assertNotIn('before', named_args)
                self.assertNotIn('after', named_args)
                self.assertIn('middleware', named_args)



"""
(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time
import uuid

import jsonschema

from freezer_api.common import exceptions as freezer_api_exc
from freezer_api.common import json_schemas


class BackupMetadataDoc(object):
    """
    Wraps a backup_metadata dict and adds some utility methods,
    and fields
    """
    def __init__(self, user_id='', user_name='', data={}):
        self.user_id = user_id
        self.user_name = user_name
        self.backup_id = uuid.uuid4().hex
        self.data = data

    def is_valid(self):
        try:
            assert (self.backup_id != '')
            assert (self.user_id != '')
            assert (self.data['container'] != '')
            assert (self.data['hostname'] != '')
            assert (self.data['backup_name'] != '')
        except Exception:
            return False
        return True

    def serialize(self):
        return {'backup_id': self.backup_id,
                'user_id': self.user_id,
                'user_name': self.user_name,
                'backup_metadata': self.data}


class JobDoc(object):
    job_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.job_schema)
    job_patch_validator = jsonschema.Draft4Validator(
        schema=json_schemas.job_patch_schema)

    @staticmethod
    def validate(doc):
        try:
            JobDoc.job_doc_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def validate_patch(doc):
        try:
            JobDoc.job_patch_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def create_patch(doc):
        # changes in user_id or job_id are not allowed
        doc.pop('user_id', None)
        doc.pop('job_id', None)
        JobDoc.validate_patch(doc)
        return doc

    @staticmethod
    def create(doc, user_id):
        job_schedule = doc.get('job_schedule', {})
        job_schedule.update({
            'time_created': int(time.time()),
            'time_started': -1,
            'time_ended': -1
        })
        doc.update({
            'user_id': user_id,
            'job_id': uuid.uuid4().hex,
            'job_schedule': job_schedule
        })
        JobDoc.validate(doc)
        return doc

    @staticmethod
    def update(doc, user_id, job_id):
        doc.update({
            'user_id': user_id,
            'job_id': job_id
        })
        JobDoc.validate(doc)
        return doc


class ActionDoc(object):
    action_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.action_schema)
    action_patch_validator = jsonschema.Draft4Validator(
        schema=json_schemas.action_patch_schema)

    @staticmethod
    def validate(doc):
        try:
            ActionDoc.action_doc_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def validate_patch(doc):
        try:
            ActionDoc.action_patch_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def create_patch(doc):
        # changes in user_id or action_id are not allowed
        doc.pop('user_id', None)
        doc.pop('action_id', None)
        ActionDoc.validate_patch(doc)
        return doc

    @staticmethod
    def create(doc, user_id):
        action_id = doc.get('action_id', uuid.uuid4().hex)
        doc.update({
            'user_id': user_id,
            'action_id': action_id,
        })
        ActionDoc.validate(doc)
        return doc

    @staticmethod
    def update(doc, user_id, action_id):
        doc.update({
            'user_id': user_id,
            'action_id': action_id,
        })
        ActionDoc.validate(doc)
        return doc


class SessionDoc(object):
    session_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.session_schema)
    session_patch_validator = jsonschema.Draft4Validator(
        schema=json_schemas.session_patch_schema)

    @staticmethod
    def validate(doc):
        try:
            SessionDoc.session_doc_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def validate_patch(doc):
        try:
            SessionDoc.session_patch_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def create_patch(doc):
        # changes in user_id or session_id are not allowed
        doc.pop('user_id', None)
        doc.pop('session_id', None)
        SessionDoc.validate_patch(doc)
        return doc

    @staticmethod
    def create(doc, user_id, hold_off=30):
        doc.update({
            'user_id': user_id,
            'session_id': uuid.uuid4().hex,
            'session_tag': doc.get('session_tag', 0),
            'status': 'active',
            'last_start': '',
            'jobs': []
        })
        doc['hold_off'] = doc.get('hold_off', hold_off)
        SessionDoc.validate(doc)
        return doc

    @staticmethod
    def update(doc, user_id, session_id):
        doc.update({
            'user_id': user_id,
            'session_id': session_id,
        })
        SessionDoc.validate(doc)
        return doc


class ClientDoc(object):
    client_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.client_schema)

    @staticmethod
    def validate(doc):
        try:
            ClientDoc.client_doc_validator.validate(doc)
        except Exception as e:
            raise freezer_api_exc.BadDataFormat(str(e).splitlines()[0])

    @staticmethod
    def create(doc, user_id):
        if 'uuid' not in doc:
            doc.update({
                'uuid': uuid.uuid4().hex
            })
        doc = {
            'client': doc,
            'user_id': user_id
        }
        ClientDoc.validate(doc)
        return doc
